#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq
from service_db import Service_DB

class Api_DB(object):
    class Error(Exception):
        pass

    def __init__(self, service_config):

        broker_database_endpoint = service_config["broker_database_endpoint"]
        self.mongodb_ip = service_config["mongodb_ip"]
        self.mongodb_port = service_config["mongodb_port"]

        context = zmq.Context()

        # adaptador de conexão
        self.socket_frontend = context.socket(zmq.REP)
        self.socket_frontend.connect(broker_database_endpoint)

        self.service = None

    def operation(self):
        # Recebe do mensagem do servidor cliente
        message = self.socket_frontend.recv()
        self.process(message)

        # Processa a operacao e envia mensage de retorno para o cliente
        self.socket_frontend.send(self.service.send())

    def run(self):
        while True:
            try:
                self.operation()
            except Api_DB.Error:
                pass

    def process(self, message):
        self.service = Service_DB(message, self.mongodb_ip, self.mongodb_port)


    def get_console_service_list(self, server_console, service_id):
        return server_console[service_id]


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .api import Api
from .api_db import Api_DB

from .error import Error
from .broker import Broker
from .datarest import DataRest
from .datarest_db import DataRest_DB

from .database import DataBase
from .filter import Filter

from .service import Service
from .service_db import Service_DB

from .datatypes import DataTypes
from .validate import Validate






#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq
from service import Service

class Api(object):
    class Error(Exception):
        pass

    def __init__(self, service_config):

        self.actions = service_config["actions"]
        self.id = service_config["id"]
        self.name = service_config["name"]
        self.schemes = service_config["schemes"]

        self.broker_rest_endpoint = service_config["broker_rest_endpoint"]
        self.broker_db_endpoint = service_config["broker_database_endpoint"]
        context = zmq.Context()

        # adaptador de conexão broker
        self.socket_broker_rest = context.socket(zmq.REP)
        self.socket_broker_rest.connect(self.broker_rest_endpoint)

        # adaptador de conexão worker
        self.socket_broker_database = context.socket(zmq.REQ)
        self.socket_broker_database.connect(self.broker_db_endpoint)

        self.service = None

    def operation(self):
        print 'self.broker_rest_endpoint', self.broker_rest_endpoint
        # Recebe do mensagem do servidor cliente
        message = self.socket_broker_rest.recv()
        self.process(message)
        self.service.action()

        # Verifica se o servico esta pronto
        if self.service.ready():
            print "SELF.BROKER_REST_ENDPOINT", self.broker_rest_endpoint
            print "SELF.BROKER_DB_ENDPOINT", self.broker_db_endpoint

            # Envia mensagem para o servidor de banco de dados
            self.socket_broker_database.send(self.service.send_database())

            # Recebe mensagem do servidor de banco de dados
            # Processa a operacao e envia mensage de retorno para o cliente
            msg = self.socket_broker_database.recv()

            self.socket_broker_rest.send(msg)

        # Em caso de falha
        else:
            # Nao processa a operação e envia mensagem de retorno para o cliente
            self.socket_broker_rest.send(self.service.send_error())

    def run(self):
        while True:
            try:
                self.operation()
            except Api.Error:
                pass

    def process(self, message):
        self.service = Service(message, self.id, self.name, self.schemes, self.actions, self.custom_service)

    def get_console_service_list(self, server_console, service_id):
        print "SERVICE_ID", service_id
        print "SERVER_CONSOLE", server_console
        return server_console[service_id]


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import datetime
import hashlib
from bson.json_util import dumps
from bson.json_util import loads
from devdoo.validate import Validate
from datatypes import DataTypes

#TODO: Verificar necessidade de uso da data_rest, transferir para a datarest e remover
class DataRest():
    def __init__(self, message):
        self.__data_types = DataTypes()
        self.__validate = Validate()
        self.decode(message)

    # --------------------------------
    # add_error
    # --------------------------------
    def add_error(self, code, message):
        self.__validate.add_error(code, message)


    # --------------------------------
    # add_error_required
    # --------------------------------
    def add_error_required(self, code, message):
        print "ERERERERE____", message
        self.__validate.add_error_required(code, message)

    # --------------------------------
    # changed_time
    # --------------------------------
    def changed_time(self):
        self.info["changed_time"] = self.__data_types.toStr(datetime.datetime.utcnow())

    # --------------------------------
    # checksum
    # --------------------------------
    def checksum(self, data):
        md5 = hashlib.md5()
        md5.update(dumps(data))
        self.info["checksum"] = md5.hexdigest()

    # --------------------------------
    # collection
    # --------------------------------
    def collection(self, collection):
        if collection != self.__service:
            list_names = collection.split("_")
            if list_names[0] != self.__service:
                list_names.insert(0, self.__service)
                collection = '_'.join(list_names)

        self.info["database"]["collection"] = collection;
        self.collection_name = collection
        self.__source()

    # --------------------------------
    # created_time
    # --------------------------------
    def created_time(self):
        self.info["created_time"] = self.__data_types.toStr(datetime.datetime.utcnow())

    # --------------------------------
    # owner_id
    # --------------------------------
    def owner_id(self):
        self.info["owner"] = {
            "id": self.token
        }

    # --------------------------------
    # owner_last_id
    # --------------------------------
    def owner_last_id(self):
        self.info["owner"] = {
            "last_id": self.token
        }

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.__validate.ready()

    # --------------------------------
    # send_database
    # --------------------------------
    def send_database(self):
        message_send = {
            "action": self.action,
            "collection_name": self.collection_name,
            "cursor": self.cursor,
            "data": self.data,
            "database_name": self.database_name,
            "error": self.__validate.error_to_list(),
            "fields": self.fields,
            "filter": self.filter,
            "id": self.id,
            "limit": self.limit,
            "order": {},
            "service_id": self.database_id
        }
        print 'MESSAGE SEND :::::::::::::::::::>>>>>>>>>>>', message_send
        # Montar mensagem que será enviada para o serviço de banco de dados
        return dumps(message_send)

    # --------------------------------
    # send_error
    # --------------------------------
    def send_error(self):
        message_send = {
            "id": self.id,
            "success": False,
            "error": self.__validate.error_to_list()
        }
        # Montar mensagem que será enviada para o serviço de banco de dados
        return dumps(message_send)

    # --------------------------------
    # validateService
    # --------------------------------
    def validateService(self, version, service, service_id):
        self.__version = version
        self.__service = service
        self.__service_id = service_id
        if self.__version != self.version or self.__service != self.service or self.__service_id != self.service_id:
            self.__validate.add_error("FALHA DE SERVICE", "Identificador do serviço nao valido")
            return False
        return True

    # --------------------------------
    # validateField
    # --------------------------------
    def validateField(self, field, value):
        return self.__validate.field(field, value)

    # ----------------------------------------------------------------
    #
    # Private Methods
    #
    # ----------------------------------------------------------------
    # --------------------------------
    # decode_service
    # --------------------------------
    def decode(self, message):
        # Prepara o pacote a ser enviado
        _json = loads(message)

        self.action = _json["action"]
        self.body = _json["body"]
        self.id = _json["id"]
        self.info = _json["info"]
        self.query = _json["query"]
        self.service = _json["service"]
        self.service_id = _json["service_id"]
        self.database_id = self.__data_types.toStr(int(_json["service_id"]) + 20000)
        self.token = _json["token"]
        self.version = _json["version"]

        self.group = self.info["application"]["group"]
        self.database_name = "db" + self.database_id + self.service
        self.info["database"] = {"name": self.database_name, "collection": None}

        print 'DECODE', _json

    # --------------------------------
    # __source
    # --------------------------------
    def __source(self):
        self.info["service"]["source"] = "/" + self.__version + "/" + self.group + "/" \
                                         + self.collection_name.replace("_", "/")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
from bson.json_util import dumps
from bson.json_util import loads
from bson import Int64
import bson, datetime
import unicodedata


class DataTypes():
    def __init__(self):
        pass

    # --------------------------------
    # isArray
    # --------------------------------
    def isArray(self, value):
        return type(value) == list

    # --------------------------------
    # isBoolean
    # --------------------------------
    def isBoolean(self, value):
        return type(value) == bool

    # --------------------------------
    # isNumber
    # --------------------------------
    def isNumber(self, value):
        return (type(value) == Int64) and value >= -9223372036854775808 and value <= 9223372036854775807

    # --------------------------------
    # isObject
    # --------------------------------
    # TODO:: Implementar validação recursiva para valores numéricos
    def isObject(self, value):
        return type(value) == dict

    # --------------------------------
    # isDecimal
    # --------------------------------
    def isDecimal(self, value):
        status = False
        if type(value) == dict and "$numberDecimal" in value.keys():
            status = len(value["$numberDecimal"]) <= 34
        return status

    # --------------------------------
    # toArray
    # --------------------------------
    # Converte em array todas as opções válidas recebidas
    # Retorna o valor original case o tipo não seja possível converter
    def toArray(self, value):
        value = self.toStr(value)
        # Verifica se a string está no formato array
        if re.match(r"\[([\s\S]*)\]\Z", value, re.IGNORECASE):
            try:
                value = value.replace("'", '"')
                return loads(value)
            except:
                pass
        # Caso contrário tentar converter para array
        elif re.match("""^([^,'";]+,[ ]?)+([^,'";]+)$""", value, re.IGNORECASE):
            value = value.split(",")
        return value

    # --------------------------------
    # toBoolean
    # --------------------------------
    # Converte em boolean todas as opções válidas recebidas
    # Retorna o valor original case o tipo não seja possível converter
    def toBoolean(self, value):
        value = self.toStr(value)
        if value.lower() in ["true", "1"]:
            value = True
        elif value.lower() in ["false", "0"]:
            value = False
        return value


    # --------------------------------
    # toDecimal
    # --------------------------------
    def toDecimal(self, value):
        value = self.toStr(value)
        # Remove todos os caracteres nao numericos
        value = re.sub(r'[^0-9.-]+', '', value)
        if len(value) > 0:
            value =  {"$numberDecimal": self.toStr(value)}
        return value

    # --------------------------------
    # toNumber
    # --------------------------------
    def toNumber(self, value):
        value = self.toStr(value)
        # Remove todos os caracteres nao numericos
        value = re.sub(r'[^0-9-]+', '', value)
        if len(value) > 0:
            value = Int64(long(value))
        return value
    # --------------------------------
    # toObject
    # --------------------------------
    def toObject(self, value):
        try:
            value = self.toStr(value)
            value = value.replace("'", '"')
            value = loads(value)
        except:
            pass
        return value

    # --------------------------------
    # toStr
    # --------------------------------
    def toStr(self, value):
        if (type(value) == unicode):
            value = unicodedata.normalize('NFKD', value).encode('utf-8', 'ignore')
            print "TO-STR-UNICODE", value
        else:
            print "TO-STR-STR", value
            value = str(value)
        return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='packagetest50',
      version='0.2',
      description='Test2',
      url='http://github.com/storborg/funniest',
      author='PulStar',
      author_email='pulstar@gmail.com',
      license='MIT',
      packages=['devdoo'],
      zip_safe=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import datetime
import hashlib, re
from bson.json_util import dumps
from bson.json_util import loads
from devdoo.validate import Validate
from devdoo.filter import Filter
from datatypes import DataTypes

class DataRest():
    def __init__(self, message, custom_service):
        self.__data_types = DataTypes()

        # TODO:: Implementar configuração de limites definido no console para o serviço
        self.__max_limit = 200
        self.__default_limit = 100

        self.__validate = Validate(custom_service)
        self.__filter = None
        self.__data = {}
        self.decode(message)

    def action(self, action):
        self.__action = action

    def cursor(self, cursor):
        self.__cursor = cursor

    def data(self, data):
        self.__data = data
        # Registra o checksum do documento
        self.checksum(data)

    # --------------------------------
    # fields
    # --------------------------------
    def fields(self, fields):
        # Verifica se recebeu uma lista de campos do usuário
        # Verifica se campos são válidos
        # Registra lista de campos solicitados
        if len(self.__fields) > 0:
            list_fields = []
            for item in self.__fields:
                if item in fields:
                    list_fields.append(item);
            self.__fields = list_fields
        else:
            #Registra lista de campos default
            self.__fields = fields


    def method(self):
        return self.__method

    # --------------------------------
    # __limit
    # --------------------------------
    # Define o limite de documentos retornados
    #
    # ?limit=20
    #
    def limit(self, max_limit=0):
        # Condigura o limite default do serviço
        limit = error_limit = self.__default_limit

        # Verifica se o limit da ação foi definido
        if self.__limit > 0:
            # Recupera o limite da operação para validar
            limit = error_limit = self.__limit

            # Verifica se o limite da operação maior do que o maximo permitido pelo serviço
            if limit > self.__max_limit:

                # Caso seja, então registra o limite maximo permitido
                limit = error_max_limit = self.__max_limit

                # Verifica se um limite máximo foi definido pelo desenvolvedor do serviço
                if max_limit > 0 and limit > max_limit:
                    # Caso seja, então registra o limite maximo permitido
                    error_max_limit = max_limit
                    limit = max_limit

                self.add_error(85545454, "O limite máximo (" + self.__data_types.toStr(error_max_limit) + ") foi excedido. Limite solicitado (" + self.__data_types.toStr(error_limit) + ").")

        self.__limit = limit

    def info(self):
        return {
            "owner": {
                "id": self.__token,
                "last_id": self.__token
            }
        }

    # --------------------------------
    # add_error
    # --------------------------------
    def add_error(self, code, message):
        self.__validate.add_error(code, message)

    # --------------------------------
    # add_error_required
    # --------------------------------
    def add_error_required(self, code, message):
        self.__validate.add_error_required(code, message)

    # --------------------------------
    # changed_time
    # --------------------------------
    def changed_time(self):
        self.__changed_time = self.__data_types.toStr(datetime.datetime.utcnow())

    # --------------------------------
    # checksum
    # --------------------------------
    def checksum(self, data):
        md5 = hashlib.md5()
        md5.update(dumps(data))
        self.__checksum = md5.hexdigest()

    # --------------------------------
    # collection
    # --------------------------------
    def collection(self, collection):
        if collection != self.__service:
            list_names = collection.split("_")
            if list_names[0] != self.__service:
                list_names.insert(0, self.__service)
                collection = '_'.join(list_names)
        self.__collection = collection

    # --------------------------------
    # created_time
    # --------------------------------
    def created_time(self):
        self.__created_time = self.__data_types.toStr(datetime.datetime.utcnow())

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.__validate.ready()

    # --------------------------------
    # send_database
    # --------------------------------
    def send_database(self):
        message = {
            "action": self.__action,
            "collection": self.__collection,
            "connection_id": self.__connection_id,
            "cursor": self.__cursor,
            "data": self.__data,
            "database": self.__database,
            "errors": self.__validate.error_to_list(),
            "fields": self.__fields,
            "filter": self.__filter.to_list(),
            "limit": self.__limit,
            "offset": self.__offset,
            "method": self.__method,
            "result_type": self.__result_type,
            "sort": self.__sort,
            "service_id": self.__database_id,
            "info": self.info()
        }

        print "SEND SAVE", message

        # Montar mensagem que será enviada para o serviço de banco de dados
        return dumps(message)

    # --------------------------------
    # send_error
    # --------------------------------
    # Montar mensagem de erro para ser retornada ao servidor rest
    def send_error(self):
        print "----------------------------AQUI"
        return dumps({
            "connection_id": self.__connection_id,
            "message": {
                "success": False,
                "message": "Falha de interação com microserviço",
                "elapsed_time": {"response": "17.000", "server": "7.000"},
                "errors": self.__validate.error_to_list(),
                "query": {
                    "limit": self.__limit,
                    "offset": self.__offset,
                    "fields": self.__fields,
                    "filter": self.__filter.to_list()  # TODO: Preparar filtro para ser retornado ao cliente
                },
            }
        })

    # --------------------------------
    # validateService
    # --------------------------------
    def validateService(self, service, service_id):
        print "SERVICE", service
        print "SERVICE_ID", service_id
        # TODO:: Implementar verificação de possivel conflito de unicode em situações não claramente definidas

        if self.__service != service or self.__data_types.toStr(self.__service_id) != self.__data_types.toStr(service_id):
            self.__validate.add_error("FALHA DE SERVICE", "Identificador do serviço nao valido:: " + self.__service + "!=" + service + "::" + self.__data_types.toStr(
                self.__service_id) + "!=" + self.__data_types.toStr(service_id))
            return False
        return True

    # --------------------------------
    # validate_field_save
    # --------------------------------
    def validate_field_save(self, item_scheme_name, item_scheme, value, action=None):
        return self.__validate.field_save(item_scheme_name, item_scheme, value, action)

    # --------------------------------
    # validate_field_update
    # --------------------------------
    def validate_field_update(self, item_scheme_name, item_scheme, value, action=None):
        return self.__validate.field_update(item_scheme_name, item_scheme, value, action)

    # ----------------------------------------------------------------
    #
    # Private Methods
    #
    # ----------------------------------------------------------------
    # --------------------------------
    # decode_service
    # --------------------------------
    def decode(self, message):
        # Prepara o pacote a ser enviado
        json = loads(message)

        print "DECODE", json

        self.body = json["body"]

        self.__action = json["action"]
        self.__connection_id = json["connection_id"]
        self.__cursor = json["cursor"]
        self.__fields = json["fields"]
        self.__filter = Filter(json["filter"], self.__validate)  # TODO:: Mudar sistema de error, tirar de dentro do validate
        self.__limit = int(json["limit"])
        self.__info = json["info"]
        self.__method = json["method"]
        self.__offset = int(json["offset"])
        self.__regex = json["regex"]
        self.__search = json["search"]
        self.__service = json["service"]
        self.__service_id = int(json["service_id"])
        self.__sort = json["sort"]
        self.__source = self.__set_source(json["source"])
        self.__token = json["token"]
        self.__version = json["version"]

        # TODO:: Mudar regra de criação de nomes de banco de dados
        self.__database_id = self.__service_id + 20000
        # self.group = self.info["application"]["group"]
        self.__database = "db" + self.__data_types.toStr(self.__database_id) + self.__service
        # self.info["database"] = {"name": self.database_name, "collection": None}

    # --------------------------------
    # __set_source
    # --------------------------------
    # Recupera o id do documento
    def __set_source(self, source):
        # Verificar se o source possui uma id
        # Pegar a id do source
        result = re.search(r"([\s\S]+)([0-9a-f]{24})$", source)
        if result:
            source = result.group(1) + ":id"
            self.__filter.add("_id.eq", result.group(2))

        return source

    # Todo Finalizar sistema de seach e regex
    def __filter_extras(self):
        # self.__prepareRegex()
        self.__prepareSearch()
        print "REGEX--------------------->>>", self.__regex
        print "SEARCH--------------------->>>", self.__search
        print "FIELDS--------------------->>>", self.__fields

    def __prepareRegex(self):
        print self.__regex

    # { name: { $regex: "s3", $options: "si" } }
    # or{quantity.lt(20)|price(10)}
    # tags.in( '{ $regex: "s3", $options: "si" }', '{ $regex: "s3", $options: "si" }')
    def __prepareSearch(self):
        values, fields = self.__search.split(';')
        values = values.split()
        fields = fields.split(',')
        if len(fields) > 0:
            for item in fields:
                print "---->", item
                for word in values:
                    # self.__filter.add("_id.eq", result.group(2))
                    # print "-............>", item+".in", word
                    print "------->", word
        print "FILTERS", self.__filter.to_list()

    # --------------------------------
    # __set_source
    # --------------------------------

    def source(self):
        return self.__source

    def result_type(self, type):
        self.__result_type = type



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from devdoo.data_rest import DataRest


class Services(object):
    def __init__(self, message):
        self.MAX_LIMIT = 1000
        self.__data_rest = DataRest(message)

    # ----------------------------------------------------------------
    #
    # Public Methods
    #
    # ----------------------------------------------------------------
    # --------------------------------
    # action
    # --------------------------------
    def action(self):
        if hasattr(self, self.__data_rest.action) and self.__data_rest.validateService(self.version, self.name,
                                                                                       self.id):
            method = getattr(self, self.__data_rest.action)
            method()
        else:
            self.__data_rest.add_error("ACTION ERRADA", self.__data_rest.action + ":: action nao existe no servico")

    # --------------------------------
    # create
    # --------------------------------
    def create(self, collection, scheme):
        self.__data_rest.action = "save"
        self.__data_rest.collection(collection)

        scheme()

        self.__data_rest.owner_id()
        self.__data_rest.owner_last_id()

        self.__data_rest.created_time()
        self.__data_rest.changed_time()

    # --------------------------------
    # find
    # --------------------------------
    def list(self, collection, scheme, max_limit=None):
        self.__max_limit = max_limit
        self.__data_rest.action = "find"
        self.__data_rest.collection(collection)

        scheme()

    # --------------------------------
    # remove
    # --------------------------------
    def remove(self, collection):
        self.__data_rest.action = "remove"
        self.__data_rest.collection(collection)

        self.scheme()

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.__data_rest.ready()

    # --------------------------------
    # scheme
    # --------------------------------
    def scheme(self, scheme):

        self.__query(scheme)

        if self.__data_rest.action == "save":
            # Recupera os dados para salvar na base de dados
            body = self.__data_rest.body
            field_body = {}
            save_data = {}

            # Processa todos os campos de body para colocar em letra minuscula e criar lista de campos validos
            for item in body:
                field = item.replace(' ', '_').lower().strip()
                field_body[field] = body[item]

                # Caso o campo nao esteja registrado no scheme, registra mensagem de error
                if field not in self.__default_fields:
                    self.__data_rest.add_error(125489, "O campo '" + field + "' nao existe.")

            # Processa todos os campos registrados no schema para validar os valores e formatar
            for item in scheme:
                value = '__empty__'
                if (item["field"]) in field_body.keys():
                    value = field_body[(item["field"])]
                    value = self.__check_empty(value)
                value = self.__data_rest.validateField(item, value)

                if value != None and value != '__empty__' and value != '__error__':
                    if value == '__none__':
                        value = None
                    save_data[(item["field"])] = value

            self.__data_rest.checksum(save_data)
            self.__data_rest.data = save_data

            # Verificar se tem algo para gravar no banco
            if len(save_data.keys()) <= 0:
                self.__data_rest.add_error_required(85545454, "Nao tem nada para gravar no banco de dados")


        elif self.__data_rest.action == "update":
            # Implementar coleção change
            # self.__data_rest.checksum(save_data)
            # TODO:: "UPDATE IMPLEMENTAR"
            print "UPDATE IMPLEMENTAR"

    # --------------------------------
    # send_database
    # --------------------------------
    def send_database(self):
        return self.__data_rest.send_database()

    # --------------------------------
    # send_error
    # --------------------------------
    def send_error(self):
        return self.__data_rest.send_error()

    # --------------------------------
    # update
    # --------------------------------
    def update(self, collection, scheme):

        self.__data_rest.action = "update"
        self.__data_rest.collection(collection)

        scheme()

        self.__data_rest.owner_last_id()
        self.__data_rest.changed_time()

    # ----------------------------------------------------------------
    #
    # Private Methods
    #
    # ----------------------------------------------------------------
    # --------------------------------
    # __fields
    # --------------------------------
    def __fields(self, query):
        fields = {}

        # TODO: Implementar erro para campos não permitidos no serviço

        # print "query.keys()", query.keys()
        # print "default_fields", self.__default_fields


        if 'fields' in query.keys():
            _fields = query["fields"]
            _fields = _fields.strip().split(',')
            for item in _fields:
                fields[item] = True

        if len(fields) == 0 and self.__default_fields != None:
            for item in self.__default_fields:
                fields[item] = True
        fields["_id"] = False
        fields["id"] = True

        return fields

    # --------------------------------
    # __filter
    # --------------------------------
    def __filter(self, query):
        filter = {}
        if 'filter' in query.keys():
            _filter = query["filter"]
            _filter = _filter.strip().split(',')

            for item in _filter:
                item = item.replace(')', "").split('(')
                filter[(item[0])] = item[1]
        return filter

    # --------------------------------
    # __empty__
    # --------------------------------
    def __check_empty(self, value):
        if (len(value) == 0 or value == ''):
            value = '__empty__'

        elif value == 'null':
            value = '__none__'

        return value

    # --------------------------------
    # __limit
    # --------------------------------
    def __limit(self, query):
        limit = self.MAX_LIMIT
        if 'limit' in query.keys():
            limit = query["limit"]
        return limit

    # --------------------------------
    # __query
    # --------------------------------
    def __query(self, scheme):
        query = self.__data_rest.query
        self.__default_fields = ["id"]
        for item in scheme:
            self.__default_fields.append(item['field'])

        # TODO:: Resolver limit
        # self.__max_limit
        # if max_limit == None:
        #    max_limit = self.MAX_LIMIT


        # TODO:: implementar opção para pegar campos de _info
        # fields["_info"] = False

        self.__data_rest.limit = self.__limit(query)
        self.__data_rest.fields = self.__fields(query)
        self.__data_rest.filter = self.__filter(query)
        self.__data_rest.order = {}
        self.__data_rest.cursor = "NDMyNzQyODI3OTQw"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from bson.json_util import dumps
from bson.json_util import loads
from devdoo.validate import Validate


class DataRest_DB():
    def __init__(self, message):
        self.__validate = Validate()
        self.decode(message)

    def action(self):
        return self.__action

    def collection(self):
        return self.__collection

    def data(self):
        return self.__data

    def database(self):
        return self.__database

    def fields(self):
        return self.__fields

    def filter(self):
        return self.__filter

    def limit(self):
        return self.__limit

    def offset(self):
        return self.__offset

    def sort(self):
        return self.__sort


    def result_type(self):
        return self.__result_type

    def info(self):
        return {
            "owner": {
                "id": self.__token,
                "last_id": self.__token
            }
        }

    # --------------------------------
    # add_error
    # --------------------------------
    def add_error(self, code, message):
        self.__validate.add_error(code, message)

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.__validate.ready()

    # --------------------------------
    # send_error
    # --------------------------------
    # TODO:: Melhorar mensagem de retorno em caso de falha
    def send_error(self):
        # Montar mensagem que será enviada para o serviço de banco de dados
        return dumps({
            "connection_id": self.__connection_id,
            "errors": self.__validate.error_to_list()
        })

    # --------------------------------
    # send_service
    # --------------------------------
    def send(self):
        print "SEND RESULT----->>>>>>>", {
            "connection_id": self.__connection_id,
            "XXXXXXXXXXX":">>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<",
            "message": self.__message_send
        }, "<<<<<-------------"

        # Registrar no stack de log
        # Registrar no stack de analytic
        # Registrar no stack de error
        # Registrar no stack de history
        # self.analyric.add({})

        return dumps({
            "connection_id": self.__connection_id,
            "message": self.__message_send
        })


    # ----------------------------------------------------------------
    #
    # Private Methods
    #
    # ----------------------------------------------------------------
    # --------------------------------
    # decode_service_db
    # --------------------------------
    def decode(self, message):
        # Prepara o pacote a ser enviado
        json = loads(message)

        self.__action = json["action"]
        self.__collection = json["collection"]
        self.__connection_id = json["connection_id"]
        self.__cursor = json["cursor"]
        self.__data = json["data"]
        self.__database = json["database"]
        self.__fields = json["fields"]
        self.__result_type = json["result_type"]
        self.__sort = json["sort"]
        self.__service_id = json["service_id"]
        self.__info = json["info"]
        self.__filter = json["filter"]
        self.__limit = json["limit"]
        self.__offset = json["offset"]
        self.__method = json["method"]
        self.__validate.add_errors(json["errors"])

        print "RECEBE SERVICE", json

        print "FILTER--------------------->>>>>", self.__filter,


    # --------------------------------
    # result
    # --------------------------------
    def result(self, message):
        message["elapsed_time"] = {"response": "17.000", "server": "7.000"}
        message["method"] = self.__method

        if self.__validate.has_error():
            message["errors"] = self.__validate.error_to_list()

        self.__message_send = message



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from bson.json_util import dumps
import requests

class Error():
    def __init__(self):
        self.has_error = False
        self.ready = True
        self.__list_errors = []

    def add(self, code, message):
        self.has_error = True
        self.__list_errors.append(
            {"code": code, "message": message}
        )
    def add_required(self, code, message):
        self.has_error = True
        self.ready = False
        self.__list_errors.append(
            {"code": code, "message": message}
        )
    def add_field(self, field, value, code, message):
        self.has_error = True
        self.__list_errors.append(
            {"field": field, "code": code, "value": value, "message": message}
        )
    def add_field_required(self, field, code, message):
        self.has_error = True
        self.ready = False
        self.__list_errors.append(
            {"field": field, "code": code, "message": message}
        )

    def to_list(self):
        if len(self.__list_errors) > 0:
            return self.__list_errors
        return None


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json, datetime, decimal
import pydash, re
from error import Error
import bson, datetime
from bson.decimal128 import Decimal128
from devdoo.datatypes import DataTypes


# TODO:: Verificar tipos inexistentes

class Validate(object):
    def __init__(self, custom_service=None):
        self.__error = Error()
        self.__custom_service = custom_service
        self.__data_types = DataTypes()

    # ----------------------------------------------------------------
    #
    # Public Methods
    #
    # ----------------------------------------------------------------
    # --------------------------------
    # add_error
    # --------------------------------
    def add_error(self, code, message):
        self.__error.add(code, message)

    # --------------------------------
    # add_error
    # --------------------------------
    def add_error_required(self, code, message):
        self.__error.add_required(code, message)

    # --------------------------------
    # add_errors
    # --------------------------------
    def add_errors(self, list_errors):
        if list_errors != None:
            for error in list_errors:
                self.__error.add(error['code'], error['message'])

    # --------------------------------
    # error_to_list
    # --------------------------------
    def error_to_list(self):
        return self.__error.to_list()

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.__error.ready

    # --------------------------------
    # has_error
    # --------------------------------
    def has_error(self):
        return self.__error.has_error

    # ----------------------------------------------------------------
    #
    # types
    #
    # ----------------------------------------------------------------
    # --------------------------------
    # array
    # --------------------------------
    # Valida campo do tipo array
    #
    # type: array
    #
    def array(self, field, value):
        original_value = self.__data_types.toStr(value)

        # Converte para array
        value = self.__data_types.toArray(value)

        # Verifica se o resultado do tipo convertido é um array válido
        if self.__data_types.isArray(value):
            return value
        else:
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.add_error(4505845, "O valor do campo '" + field["field"] + "' nao pode ser '" + original_value + "', vazio ou null, utilize um valor do tipo array [\"a\", \"b\", 1, 2]")

            value = self.__check_value(value)

        return value

    # --------------------------------
    # boolean
    # --------------------------------
    # Valida campo do tipo boleano
    #
    # type: boolean
    #
    def boolean(self, field, value):
        original_value = self.__data_types.toStr(value)

        # Converte para boleano
        value = self.__data_types.toBoolean(value)

        # Verifica se o resultado do tipo convertido é um boleano válido
        if self.__data_types.isBoolean(value):
            return value
        else:
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.add_error(4505845, "O valor do campo '" + field["field"] + "' nao pode ser '" + original_value + "', vazio ou null, utilize um valor entre true | false | 0 | 1")
            value = self.__check_value(value)

        return value

    # --------------------------------
    # date
    # --------------------------------
    # Valida campo do tipo data
    def date(self, field, value):
        # 26 / 10 / 2017
        # 2017 - 10 - 26
        '''
        try:
            value = datetime.datetime.strptime(value, '%Y-%m-%d')
        except ValueError:
            value = self.__register_error(field, value, "DATE")
        '''
        return value

    # --------------------------------
    # enum
    # --------------------------------
    # Valida campo do tipo opções de enumeração
    # Aceita somente opções string
    #
    # type: enum
    # enum:["M", "F", "34"]
    # is_required = self.__required(item_scheme)
    #
    def enum(self, item_scheme, value):
        options = self.__type(item_scheme, "enum")

        # Converte para string
        value = self.__data_types.toStr(value)

        if value == '__empty__' or value == '__none__':
            value = self.__clean_value(value)
            self.add_error(4505845, "O valor do campo '" + item_scheme[
                "field"] + "' nao pode ser '" + value + "', vazio ou null, utilize um valor entre: " + str(options))
            value = '__error__'

        if value not in options:
            self.add_error(4505845, "O valor do campo '" + item_scheme[
                "field"] + "' :: '" + value + "', deve conter na lista de opcoes: " + str(options))
            value = '__error__'

        return value

    # --------------------------------
    # mixed
    # --------------------------------
    # Valida campo do tipo mixed, aceita qualquer valor no campo
    #
    # type: mixed
    #
    def mixed(self, field, value):
        # aceita qualquer valor
        return value

    # --------------------------------
    # number
    # --------------------------------
    # Valida campo do tipo número longo
    #
    # type: number
    # min_value: 150
    # max_value: 300
    # formater:??
    #
    def number(self, field, value):
        original_value = self.__data_types.toStr(value)

        # Converte valor para número
        value = self.__data_types.toNumber(value)

        # Verifica se o resultado do tipo convertido é um boleano válido
        if self.__data_types.isNumber(value):
            # min_value / max_value
            error_min = self.__min_value(field, value)
            error_max = self.__max_value(field, value)
            if error_min or error_max:
                value = '__error__'
        else:
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.add_error(8545255, "O campo '" + field["field"] + "' nao pode ser '" + original_value + "', vazio ou null, utilize um valor entre -x até x.")
            value = self.__check_value(value)

        return value

    # --------------------------------
    # number_decimal
    # --------------------------------
    # Valida campo do tipo númro decimal
    def decimal(self, field, value):
        original_value = self.__data_types.toStr(value)

        # Converte valor para número
        value = self.__data_types.toDecimal(value)

        # Verifica se o resultado do tipo convertido é um boleano válido
        if self.__data_types.isDecimal(value):
            pass
            #TODO:: Implementar min/max para decimal
            # min_value / max_value
            #error_min = self.__min_value_decimal(field, value)
            #error_max = self.__max_value_decimal(field, value)
            #if error_min or error_max:
            #    value = '__error__'
        else:
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.add_error(8545255,
                           "O campo '" + field["field"] + "' nao pode ser '" + original_value + "', vazio ou null, utilize um valor entre -x até x.")
            value = self.__check_value(value)

        return value

    # --------------------------------
    # object
    # --------------------------------
    # Valida campo do tipo object
    #
    # type: object
    #
    def object(self, field, value):
        original_value = self.__data_types.toStr(value)

        # Converte valor para número
        value = self.__data_types.toObject(value)

        # Verifica se o resultado do tipo convertido é objeto válido
        if not self.__data_types.isObject(value):
            self.add_error(4505845, "O valor do campo '" + field[
                "field"] + "' nao pode ser '" + original_value + "', vazio ou null, utilize um valor do tipo object")
            value = '__error__'

        return value

    # --------------------------------
    # object_id
    # --------------------------------
    # Valida campo do tipo objectId
    #
    # type: object_id
    #
    def object_id(self, field, value):
        try:
            value = str(value)
            value = bson.objectid.ObjectId(value)
        except:
            value = self.__clean_value(value)

            self.add_error(4505845, "O valor do campo '" + field[
                "field"] + "' nao pode ser '" + str(value) + "', vazio ou null, utilize um valor do tipo object_id")
            value = '__error__'

        return value

    # --------------------------------
    # string
    # --------------------------------
    #
    # Valida campo do tipo string
    #
    # type: string
    # modifier: ["uppercase", "..."]
    # min_length: 5
    # max_length: 15
    # formater:??
    #
    def string(self, field, value):
        # Remove elementos indesejados da string
        value = pydash.strings.escape(value).strip()

        value = self.__clean_value(value)

        # min_length / max_length
        error_min = self.__min_length(field, value)
        error_max = self.__max_length(field, value)
        if error_min or error_max:
            value = '__error__'
            return value

        # Implementa modificadores
        value = self.__modifier(field, value)

        return value

    # --------------------------------
    # timestamp
    # --------------------------------
    # Valida campo do tipo timestamp
    def timestamp(self, field, value):
        if str(value) == 'now':
            value = bson.timestamp.Timestamp(datetime.datetime.utcnow(), 0)
            return value
        try:
            value = re.sub(r'[^0-9]+', '', str(value))
            value = bson.timestamp.Timestamp(int(value), 0)
        except:
            value = self.__clean_value(value)

            self.add_error(4505845, "O valor do campo '" + field[
                "field"] + "' nao pode ser '" + str(value) + "', vazio ou null, utilize um valor do tipo timestamp")
            value = '__error__'
        return value

    # ----------------------------------------------------------------
    #
    # Init Process
    #
    # ----------------------------------------------------------------
    # --------------------------------
    # field
    # --------------------------------
    def __field(self, item_scheme_name, item_scheme, value, action=None):
        # Implementa valor default em caso do campo não receber valor do cliente
        field_name = item_scheme_name

        field_value = self.__default(item_scheme, value)
        field_type = item_scheme["type"]

        if hasattr(self, field_type):
            # Recupera o método do type que deverá ser executado
            method = getattr(self, field_type)
            field_value = method(item_scheme, field_value)
        else:
            field_value == '__error__'
            self.add_error(854857, "O tipo do campo '" + item_scheme["field"] + " -> " + field_type + "' nao existe.")
            return field_value

        # Verifica se o campo e requerido
        if field_value == '__empty__' or field_value == '':
            self.__required(item_scheme)
        # if action != None:
        #    field_name, field_value = self.__custom(action, item_scheme, field_value, self.__error)

        return field_name, field_value

    def field_save(self, item_scheme_name, item_scheme, value, action=None):
        field_name, field_value = self.__field(item_scheme_name, item_scheme, value, action=None)
        # Verifica se o nome do campo é dot
        if "." in field_name:
            field_name, field_value = self.__field_dot(field_name, field_value)

        return field_name, field_value

    def field_update(self, item_scheme_name, item_scheme, value, action=None):
        return self.__field(item_scheme_name, item_scheme, value, action=None)

    # ----------------------------------------------------------------
    #
    # Private Methods
    #
    # ----------------------------------------------------------------
    # --------------------------------
    # __custom
    # --------------------------------
    def __custom(self, action, item_scheme, field_value, error):
        field_name = item_scheme["field"]

        if action in item_scheme.keys():
            # Verifica se a acao existe na instancia do servico
            if hasattr(self.__custom_service, item_scheme["field"] + "_" + action):
                method = getattr(self.__custom_service, item_scheme["field"] + "_" + action)
                field_name, field_value = method(field_name, field_value, error)

        return field_name, field_value

    # --------------------------------
    # __default
    # --------------------------------
    def __default(self, field, value):
        if value == '__empty__' or value == None:
            if "default" in field.keys():
                value = field["default"]
        return value

    # --------------------------------
    # __clean_value
    # --------------------------------
    # TODO:: Remover metodo
    def __clean_value(self, value):
        if value == '__empty__' or value == '__none__':
            value = ''
        return value

    # --------------------------------
    # __check_empty
    # --------------------------------
    def __check_empty(self, value):
        if (len(value) == 0 or value == ''):
            value = '__empty__'

        elif value == 'null':
            value = '__none__'

        return value

    # --------------------------------
    # __check_value
    # --------------------------------
    def __check_value(self, value):
        if value == '__empty__' or value == '__none__':
            value = ''
        else:
            value = '__error__'

        return value

    # --------------------------------
    # __lodash
    # --------------------------------
    def __lodash(self, action, value):
        if action == 'camel_case':
            value = pydash.strings.camel_case(value)
        elif action == 'clean':
            value = pydash.strings.clean(value)
        elif action == 'decapitalize':
            value = pydash.strings.decapitalize(value)
        elif action == 'escape':
            value = pydash.strings.escape(value)
        elif action == 'lowercase':
            value = pydash.strings.to_lower(value)
        elif action == 'kebab_case':
            value = pydash.strings.kebab_case(value)
        elif action == 'uppercase':
            value = pydash.strings.to_upper(value)
        elif action == 'upper_first':
            value = pydash.strings.upper_first(value)
        elif action == 'startcase':
            value = pydash.strings.start_case(value)
        elif action == 'titlecase':
            value = pydash.strings.title_case(value)
        elif action == 'unescape':
            value = pydash.strings.unescape(value)

        return value

    # --------------------------------
    # __max_length
    # --------------------------------
    def __max_length(self, field, value):
        has_error = False
        max_length = self.__type(field, "max_length")
        if max_length != None and len(value) > max_length:
            message = "O tamanho máximo permitido é (" + self.__data_types.toStr(max_length) + "), foi recebido (" + self.__data_types.toStr(
                len(value)) + ")."
            self.__register_error(field, value, "MAX LENGTH", message)
            has_error = True
        return has_error

    # --------------------------------
    # __min_length
    # --------------------------------
    def __min_length(self, field, value):
        has_error = False
        min_length = self.__type(field, "min_length")
        if min_length != None and len(value) < min_length:
            message = "O tamanho mínimo permitido é (" + self.__data_types.toStr(min_length) + "), foi recebido (" + self.__data_types.toStr(
                len(value)) + ")."
            self.__register_error(field, value, "MIN LENGTH", message)
            has_error = True
        return has_error

    # --------------------------------
    # __max_value
    # --------------------------------
    def __max_value(self, field, value):
        has_error = False
        max_value = self.__type(field, "max_value")
        if max_value and value > bson.Int64(long(max_value)):
            message = "O valor máximo permitido é (" + self.__data_types.toStr(max_value) + "), foi recebido (" + self.__data_types.toStr(
                value) + ")."
            self.__register_error(field, value, "MAX VALUE", message)
            has_error = True
        return has_error

    # --------------------------------
    # __min_value
    # --------------------------------
    def __min_value(self, field, value):
        has_error = False
        min_value = self.__type(field, "min_value")
        if min_value and value < bson.Int64(long(min_value)):
            message = "O valor mínimo permitido é (" + self.__data_types.toStr(min_value) + "), foi recebido (" + self.__data_types.toStr(
                value) + ")."
            self.__register_error(field, value, "MIN VALUE", message)
            has_error = True
        return has_error

    # --------------------------------
    # __modifier
    # --------------------------------
    def __modifier(self, field, value):
        modifier = self.__type(field, "modifier")
        if modifier != None:
            for action in modifier:
                value = self.__lodash(action, value)
        return value

    # --------------------------------
    # __options
    # --------------------------------
    def __options(self):
        pass

    # --------------------------------
    # __register_error
    # --------------------------------
    def __register_error(self, scheme, value, code, message=None):
        self.__error.add_field(scheme["field"], value, code, message)
        return '__error__'

    # --------------------------------
    # __required
    # --------------------------------
    def __required(self, item_scheme):
        is_required = self.__type(item_scheme, "required")
        if is_required:
            self.__error.add_field_required(item_scheme['field'], "REQUIRED", "Campo requerido")
        return is_required

    # --------------------------------
    # __type
    # --------------------------------
    def __type(self, scheme, field):
        result = None
        if field in scheme.keys():
            result = scheme[field]
        return result

    # --------------------------------
    # __validate
    # --------------------------------
    def __validate(self, value):
        pass

    def __field_dot(self, field_name, field_value):
        d = {}

        field_name_first, field_name = field_name.split(".", 1)

        # Pegar a primeira chave do nome campo
        # Pagar as partes restantes e converter em objeto com valor

        # Converte string em objetos
        def put(d, dot_string, value):
            if "." in dot_string:
                key, rest = dot_string.split(".", 1)
                if key not in d:
                    d[key] = {}
                put(d[key], rest, value)
            else:
                d[dot_string] = value

        put(d, field_name, field_value)
        return field_name_first, d



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq
from bson.json_util import loads
from datatypes import DataTypes

class Broker(object):
    class Error(Exception):
        pass

    def __init__(self, broker_id, console_service_list):
        self.__data_types = DataTypes()
        context = zmq.Context()


        # Initialize poll set
        self.poller = zmq.Poller()

        # Abre porta para o Restify (frontend)
        self.socket_frontend = context.socket(zmq.ROUTER)
        self.socket_frontend.bind("tcp://*:" + self.__data_types.toStr(broker_id))
        self.poller.register(self.socket_frontend, zmq.POLLIN)

        # Abre portas para os services
        self.dicServices = {}
        for service_item in console_service_list:
            service_id = self.__data_types.toStr(service_item["service_id"])
            endpoint = "tcp://*:" + service_id

            # Abre portas para os Services (backend)
            socket_backend = context.socket(zmq.DEALER)

            socket_backend.bind(endpoint)

            # TODO:: Implementar Lista de serviços do mesmo tipo
            self.dicServices[service_id] = socket_backend

        self.socket_backend = None

    def operation(self):
        socks = dict(self.poller.poll())

        if socks.get(self.socket_frontend) == zmq.POLLIN:
            # Recebe mensagem do Restify
            message = self.socket_frontend.recv_multipart()

            # Converte mensagem para json
            package = loads(message[2])
            print "BROKER PACKAGE", package

            # Recupera o socket de conexao do serviço
            self.socket_backend = self.dicServices[self.__data_types.toStr(package["service_id"])]
            print "BROKER: MESSAGE", self.__data_types.toStr(package["service_id"])

            # Verifica se o socket é valiso
            if self.socket_backend:
                # Registra o socket no poller ZMQ para dar acesso aos workers
                self.poller.register(self.socket_backend, zmq.POLLIN)

                # Envia ao Service Work a mensagem com pacote de dados
                self.socket_backend.send_multipart(message)
            else:
                # Caso contrario retorna para o cliente sem realizar nenhuma alteração
                self.socket_frontend.send_multipart(message)

        # Verifica se o socket retornou do Service Worker
        if socks.get(self.socket_backend) == zmq.POLLIN:
            # Recebe do Service Worker
            message = self.socket_backend.recv_multipart()


            print "OPOPOPOPOPOPOPOPOPOPOPO", message
            # Retorna ao Restify
            self.socket_frontend.send_multipart(message)
            self.poller.unregister(self.socket_backend)

    def run(self):
        while True:
            try:
                self.operation()
            except Broker.Error:
                pass

    def process(self, data):
        raise Broker.Error("`process` should be overriden")
